import matplotlib
matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
import glob
import os
import sys
from pylal.xlal.datatypes.ligotimegps import LIGOTimeGPS
from pylal.xlal.date import XLALGreenwichSiderealTime
import find_trig as ft
import healpy as hp
from subprocess import call
# from scipy.optimize import curve_fit
import acor
from pylal import bayespputils
import sky_area.sky_area_clustering as sky
import skyview
from scipy import stats

# --------------
# Parameters
# --------------
# -- Set skymap resolution
# NSIDE = 32
# mdcfile = '/home/jkanner/baysewave/svn/trunk/burstinj/s6/BurstMDC-BRST_S6-Log.txt'

# -- Close any open figures
plt.close('all')

# ----------------
# Read S6 MDC log file
# -----------------
# print "Reading MDC log (this may take a few minutes) ..."
# mdc = ft.Mdc(filename=mdcfile)
# print "I read the mdc log!"


# -- Set up lists of results
glitch = []
signal = []
scale = []
snrList = []
goodJobList = []
skydict = {}
# injcontourList = []
# searchareaList = []
# area50List = []
# area90List = []

# -- Get directory list
topdir = os.getcwd()
dirList = glob.glob('*/job*')
# dirList = [os.path.abspath(directory) for directory in dirList]

# -- Loop over jobs
for job in dirList:
    os.chdir(job)
    print "Entered", os.getcwd()

    # --------------------------------
    # Process skymap information
    # --------------------------------
    # -- Check if skystats exists
    if os.path.isfile('skystats.txt'):
        # area50, area90, searcharea, injcontour = np.loadtxt('skystats.txt', unpack=True)
        print "Found skystats file!"
        # skydata = np.recfromtxt('skystats.txt', names='True')
        # for key in ['area50', 'area90', 'searcharea', 'injcontour']:
        #    skydict[key].append(skydata[key])

        goodJobList.append(job)
        skydata = open('skystats.txt', 'r')
        keylist = skydata.readline().split()[1:]
        valuelist = skydata.readline().split()
        for key, value in zip(keylist, valuelist):
            if key not in skydict: skydict[key] = []
            skydict[key].append(float(value))
            
    else:
        # print "No skystats found :(  Running skyview.  You have time for a coffee break ..."
        # skyview.make_skyview(mdc=mdc, NSIDE=NSIDE, results=skydict, sim=True)
        print "No skystats found :("

    # -- Return to top level directory, continue through list of jobs
    plt.close('all')
    os.chdir(topdir)

# -- Need to set the matplotlib figure size
plt.rcParams['figure.figsize'] = 8, 6

# -------------
# Make PP plot
# ------------
injconArray = np.array(skydict['injcontour'])
print injconArray
xtics = np.arange(0,1.01,0.01)
numFound = np.array( [ (injconArray <= xvalue).sum() for xvalue in xtics] )
fracFound = 1.0*numFound / injconArray.size

plt.close('all')
plt.figure()

# -- Make 1 sigma confidence region
print "attempting to calculate PP plots"
bi = stats.binom
N = len(injconArray)

# -- 1 sigma
con = 0.68
edge = (1 - con)/2.0
uppery = bi.ppf(1-edge, N, xtics)
lowery = bi.ppf(edge, N, xtics)
uppery[0]=0
lowery[0]=0
plt.fill_between(xtics, lowery/N, uppery/N, facecolor='gray', alpha=0.2)

# -- 2 sigma
con = 0.95
edge = (1 - con)/2.0
uppery = bi.ppf(1-edge, N, xtics)
lowery = bi.ppf(edge, N, xtics)
uppery[0]=0
lowery[0]=0
plt.fill_between(xtics, lowery/N, uppery/N, facecolor='gray', alpha=0.2)


# -- 3 sigma
con = 0.997
edge = (1 - con)/2.0
uppery = bi.ppf(1-edge, N, xtics)
lowery = bi.ppf(edge, N, xtics)
uppery[0]=0
lowery[0]=0
plt.fill_between(xtics, lowery/N, uppery/N, facecolor='gray', alpha=0.2)


plt.plot(xtics, fracFound)
plt.xlabel('Confidence Region')
plt.ylabel('Fraction of injections found')
plt.plot( [0, 1], [0, 1], 'k--', linewidth=2 )
plt.savefig('pp.png')
plt.close()

# -------------------
# Histogram the areas
# -------------------
bins = np.arange(0, np.log10(41253), np.log10(41253)/20)
area50 = np.array(skydict['area50'])
area90 = np.array(skydict['area90'])
searcharea = np.array(skydict['searcharea'])
plt.figure()
plt.hist(np.log10(area50), bins=bins)
plt.xlabel('log10 Area for 50% Confidence')
plt.ylabel('Number of injections')
plt.savefig('area50.png')
plt.close()

plt.figure()
plt.hist(np.log10(area90), bins=bins)
plt.xlabel('log10 Area for 90% Confidence')
plt.ylabel('Number of injections')
plt.savefig('area90.png')
plt.close()

plt.figure()
plt.hist(np.log10(searcharea), bins=bins)
plt.xlabel('log10 Search area')
plt.ylabel('Number of injections')
plt.savefig('searcharea.png')
plt.close()


# -------------------------
# Try making sky area table
# -------------------------
skyTable = open('skyTable.html', 'w')
skyTable.write('<html><body>')
skyTable.write('<h1>Bayes Wave Burst Sky Table</h1>')
skyTable.write('<img src="./pp.png" width=600>')
skyTable.write('<img src="./area50.png" width=600>')
skyTable.write('<img src="./area90.png" width=600>')
skyTable.write('<img src="./searcharea.png" width=600>')
skyTable.write('<h3>List of events</h3>')
skyTable.write('<table>')
skyTable.write('<tr><b><td> Job    <td>50% Area  <td>90% Area <td>Found Contour</b></tr>')

# -- Make tuple, sort by SNR
raw_eventList = zip(area50, area90, injconArray, goodJobList)
# raw_eventList = zip(area50, area90, injconArray, dirList)

eventList =  sorted(raw_eventList, key=lambda event: event[0], reverse=True)
for (a50, a90, injc, job) in eventList:
    skyTable.write("""<tr>
<td>{0}  
<td><a href='{1}/skyview.html'>{1}</a>  
<td>{2:.2f}  
<td>{3:.2f}
<td>{4:.2f}
</tr>
""".format('- ', job, a50, a90, injc) )
skyTable.write('</table></html></body>')
skyTable.close()
